@@ -28,13 +28,11 @@ mod blob {
                         .buffers_full_delay_ms(0)
                         .part_size_bytes(part_size)
                         .max_concurrency(n_concurrency);
-                    let condow =
-                        Condow::new_with_reporting(client.clone(), config, SimpleReporterFactory)
-                            .unwrap();
+                    let condow = Condow::new(client.clone(), config).unwrap();
 
-                    let downloader = condow.downloader();
+                    let downloader = condow.downloader_with_reporting(SimpleReporterFactory);
 
-                    let result_stream = downloader.download_rep((), ..).await.unwrap();
+                    let result_stream = downloader.download_rep(NoLocation, ..).await.unwrap();
 
                     let result = result_stream.into_stream().into_vec().await.unwrap();
 
@@ -65,13 +63,11 @@ mod blob {
                         .buffers_full_delay_ms(0)
                         .part_size_bytes(part_size)
                         .max_concurrency(n_concurrency);
-                    let condow =
-                        Condow::new_with_reporting(client.clone(), config, SimpleReporterFactory)
-                            .unwrap();
+                    let condow = Condow::new(client.clone(), config).unwrap();
 
-                    let downloader = condow.downloader();
+                    let downloader = condow.downloader_with_reporting(SimpleReporterFactory);
 
-                    let result_stream = downloader.download_rep((), ..).await.unwrap();
+                    let result_stream = downloader.download_rep(NoLocation, ..).await.unwrap();
 
                     let report = result_stream.reporter.report();
                     assert!(report.download_time > Duration::ZERO);
@@ -116,7 +112,7 @@ mod range {
 
                             let result_stream = condow
                                 .download_chunks_internal(
-                                    (),
+                                    NoLocation,
                                     range.clone(),
                                     crate::GetSizeMode::Always,
                                     SimpleReporter::default(),
@@ -162,7 +158,7 @@ mod range {
 
                             let result_stream = condow
                                 .download_chunks_internal(
-                                    (),
+                                    NoLocation,
                                     range.clone(),
                                     crate::GetSizeMode::Required,
                                     SimpleReporter::default(),
@@ -216,7 +212,7 @@ mod range {
 
                             let result_stream = condow
                                 .download_chunks_internal(
-                                    (),
+                                    NoLocation,
                                     range.clone(),
                                     crate::GetSizeMode::Default,
                                     SimpleReporter::default(),
@@ -262,7 +258,7 @@ mod range {
 
                             let result_stream = condow
                                 .download_chunks_internal(
-                                    (),
+                                    NoLocation,
                                     range.clone(),
                                     crate::GetSizeMode::Default,
                                     SimpleReporter::default(),
@@ -315,7 +311,7 @@ mod range {
 
                                     let result_stream = condow
                                         .download_chunks_internal(
-                                            (),
+                                            NoLocation,
                                             range,
                                             crate::GetSizeMode::Default,
                                             SimpleReporter::default(),
@@ -362,7 +358,7 @@ mod range {
 
                                     let result_stream = condow
                                         .download_chunks_internal(
-                                            (),
+                                            NoLocation,
                                             range,
                                             crate::GetSizeMode::Default,
                                             SimpleReporter::default(),
@@ -418,7 +414,7 @@ mod range {
 
                                         let result_stream = condow
                                             .download_chunks_internal(
-                                                (),
+                                                NoLocation,
                                                 range,
                                                 crate::GetSizeMode::Default,
                                                 SimpleReporter::default(),
@@ -468,7 +464,7 @@ mod range {
 
                                         let result_stream = condow
                                             .download_chunks_internal(
-                                                (),
+                                                NoLocation,
                                                 range,
                                                 crate::GetSizeMode::Default,
                                                 SimpleReporter::default(),
