@@ -1,38 +1,70 @@
+use std::sync::Arc;
+
+use futures::future::BoxFuture;
+
 use crate::{
     condow_client::CondowClient,
     errors::{CondowError, GetSizeError},
-    reporter::{NoReporter, Reporter, ReporterFactory},
+    reporter::{NoReporting, Reporter, ReporterFactory},
     streams::{ChunkStream, PartStream},
-    Condow, DownloadRange, GetSizeMode, Outcome,
+    Condow, DownloadRange, Downloads, GetSizeMode, StreamWithReport,
 };
 
 /// A configured downloader.
 ///
 /// This struct has state which configures a download.
-pub struct Downloader<C: CondowClient, RF: ReporterFactory = NoReporter> {
+pub struct Downloader<C: CondowClient, RF: ReporterFactory = NoReporting> {
     /// Mode for handling upper bounds of a range and open ranges
     ///
     /// Default: As configured with [Condow] itself
     pub get_size_mode: GetSizeMode,
-    condow: Condow<C, RF>,
+    condow: Condow<C>,
+    reporter_factory: Arc<RF>,
 }
 
-impl<C: CondowClient, RF: ReporterFactory> Downloader<C, RF> {
-    pub(crate) fn new(condow: Condow<C, RF>) -> Self {
-        Self {
-            condow,
-            get_size_mode: GetSizeMode::default(),
-        }
+impl<C: CondowClient> Downloader<C, NoReporting> {
+    pub(crate) fn new(condow: Condow<C>) -> Self {
+        Self::new_with_reporting(condow, NoReporting)
     }
 }
 
 impl<C: CondowClient, RF: ReporterFactory> Downloader<C, RF> {
+    pub(crate) fn new_with_reporting(condow: Condow<C>, rep_fac: RF) -> Self {
+        Self::new_with_reporting_arc(condow, Arc::new(rep_fac))
+    }
+
+    pub(crate) fn new_with_reporting_arc(condow: Condow<C>, rep_fac: Arc<RF>) -> Self {
+        Self {
+            condow,
+            get_size_mode: GetSizeMode::default(),
+            reporter_factory: rep_fac,
+        }
+    }
+
     /// Change the behaviour on when to query the file size
     pub fn get_size_mode<T: Into<GetSizeMode>>(mut self, get_size_mode: T) -> Self {
         self.get_size_mode = get_size_mode.into();
         self
     }
 
+    pub fn with_reporting<RRF: ReporterFactory>(self, rep_fac: RRF) -> Downloader<C, RRF> {
+        self.with_reporting_arc(Arc::new(rep_fac))
+    }
+
+    pub fn with_reporting_arc<RRF: ReporterFactory>(self, rep_fac: Arc<RRF>) -> Downloader<C, RRF> {
+        let Downloader {
+            get_size_mode,
+            condow,
+            ..
+        } = self;
+
+        Downloader {
+            condow,
+            get_size_mode,
+            reporter_factory: rep_fac,
+        }
+    }
+
     /// Download the BLOB/range.
     ///
     /// The parts and the chunks streamed have the same ordering as
@@ -59,7 +91,7 @@ impl<C: CondowClient, RF: ReporterFactory> Downloader<C, RF> {
         range: R,
     ) -> Result<ChunkStream, CondowError> {
         self.condow
-            .download_chunks_internal(location, range, self.get_size_mode, NoReporter)
+            .download_chunks_internal(location, range, self.get_size_mode, NoReporting)
             .await
             .map(|o| o.stream)
     }
@@ -74,8 +106,8 @@ impl<C: CondowClient, RF: ReporterFactory> Downloader<C, RF> {
         &self,
         location: C::Location,
         range: R,
-    ) -> Result<Outcome<PartStream<ChunkStream>, RF::ReporterType>, CondowError> {
-        let reporter = self.condow.reporter_factory.make();
+    ) -> Result<StreamWithReport<PartStream<ChunkStream>, RF::ReporterType>, CondowError> {
+        let reporter = self.reporter_factory.make();
         self.download_wrep(location, range, reporter).await
     }
 
@@ -91,8 +123,8 @@ impl<C: CondowClient, RF: ReporterFactory> Downloader<C, RF> {
         &self,
         location: C::Location,
         range: R,
-    ) -> Result<Outcome<ChunkStream, RF::ReporterType>, CondowError> {
-        let reporter = self.condow.reporter_factory.make();
+    ) -> Result<StreamWithReport<ChunkStream, RF::ReporterType>, CondowError> {
+        let reporter = self.reporter_factory.make();
         self.download_chunks_wrep(location, range, reporter).await
     }
 
@@ -107,7 +139,7 @@ impl<C: CondowClient, RF: ReporterFactory> Downloader<C, RF> {
         location: C::Location,
         range: R,
         reporter: RP,
-    ) -> Result<Outcome<PartStream<ChunkStream>, RP>, CondowError> {
+    ) -> Result<StreamWithReport<PartStream<ChunkStream>, RP>, CondowError> {
         self.download_chunks_wrep(location, range, reporter)
             .await?
             .part_stream()
@@ -126,7 +158,7 @@ impl<C: CondowClient, RF: ReporterFactory> Downloader<C, RF> {
         location: C::Location,
         range: R,
         reporter: RP,
-    ) -> Result<Outcome<ChunkStream, RP>, CondowError> {
+    ) -> Result<StreamWithReport<ChunkStream, RP>, CondowError> {
         self.condow
             .download_chunks_internal(location, range, self.get_size_mode, reporter)
             .await
@@ -137,3 +169,35 @@ impl<C: CondowClient, RF: ReporterFactory> Downloader<C, RF> {
         self.condow.get_size(location).await
     }
 }
+
+impl<C: CondowClient, RF: ReporterFactory> Clone for Downloader<C, RF> {
+    fn clone(&self) -> Self {
+        Self {
+            condow: self.condow.clone(),
+            reporter_factory: Arc::clone(&self.reporter_factory),
+            get_size_mode: self.get_size_mode,
+        }
+    }
+}
+
+impl<C, RF> Downloads<C::Location> for Downloader<C, RF>
+where
+    C: CondowClient,
+    RF: ReporterFactory,
+{
+    fn download<'a, R: Into<DownloadRange> + Send + Sync + 'static>(
+        &'a self,
+        location: C::Location,
+        range: R,
+    ) -> BoxFuture<'a, Result<PartStream<ChunkStream>, CondowError>> {
+        Box::pin(self.download(location, range))
+    }
+
+    fn download_chunks<'a, R: Into<DownloadRange> + Send + Sync + 'static>(
+        &'a self,
+        location: C::Location,
+        range: R,
+    ) -> BoxFuture<'a, Result<ChunkStream, CondowError>> {
+        Box::pin(self.download_chunks(location, range))
+    }
+}
