@@ -23,8 +23,8 @@ use condow_client::CondowClient;
 use config::{AlwaysGetSize, Config};
 use errors::{CondowError, GetSizeError};
 
-use futures::Stream;
-use reporter::{NoReporter, Reporter, ReporterFactory};
+use futures::{future::BoxFuture, Stream};
+use reporter::{NoReporting, Reporter, ReporterFactory};
 use streams::{BytesHint, ChunkStream, ChunkStreamItem, PartStream};
 
 #[macro_use]
@@ -44,6 +44,29 @@ pub use downloader::*;
 #[cfg(test)]
 pub mod test_utils;
 
+pub trait Downloads<L>
+where
+    L: std::fmt::Debug + std::fmt::Display + Clone + Send + Sync + 'static,
+{
+    /// Download a BLOB range (potentially) concurrently
+    ///
+    /// Returns a stream of [Chunk](streams::Chunk)s.
+    fn download<'a, R: Into<DownloadRange> + Send + Sync + 'static>(
+        &'a self,
+        location: L,
+        range: R,
+    ) -> BoxFuture<'a, Result<PartStream<ChunkStream>, CondowError>>;
+
+    /// Download a BLOB range (potentially) concurrently
+    ///
+    /// Returns a stream of [Parts](streams::Part)s.
+    fn download_chunks<'a, R: Into<DownloadRange> + Send + Sync + 'static>(
+        &'a self,
+        location: L,
+        range: R,
+    ) -> BoxFuture<'a, Result<ChunkStream, CondowError>>;
+}
+
 /// The CONcurrent DOWnloader
 ///
 /// Downloads BLOBs by splitting the download into parts
@@ -54,51 +77,47 @@ pub mod test_utils;
 /// * `Range`: A range to be downloaded of a BLOB (Can also be the complete BLOB)
 /// * `Part`: The downloaded range is split into parts of certain ranges which are downloaded concurrently
 /// * `Chunk`: A chunk of bytes received from the network (or else). Multiple chunks make a part.
-pub struct Condow<C, RF = NoReporter> {
+pub struct Condow<C> {
     client: C,
     config: Config,
-    reporter_factory: Arc<RF>,
 }
 
-impl<C: CondowClient, RF: ReporterFactory> Clone for Condow<C, RF> {
+impl<C: CondowClient> Clone for Condow<C> {
     fn clone(&self) -> Self {
         Self {
             client: self.client.clone(),
             config: self.config.clone(),
-            reporter_factory: Arc::clone(&self.reporter_factory),
         }
     }
 }
 
-impl<C: CondowClient> Condow<C, NoReporter> {
+impl<C: CondowClient> Condow<C> {
     /// Create a new CONcurrent DOWnloader.
     ///
     /// Fails if the [Config] is not valid.
     pub fn new(client: C, config: Config) -> Result<Self, anyhow::Error> {
         let config = config.validated()?;
-        Self::new_with_reporting(client, config, NoReporter)
-    }
-}
-
-impl<C: CondowClient, RF: ReporterFactory> Condow<C, RF> {
-    pub fn new_with_reporting(
-        client: C,
-        config: Config,
-        rep_fac: RF,
-    ) -> Result<Self, anyhow::Error> {
-        let config = config.validated()?;
-        Ok(Self {
-            client,
-            config,
-            reporter_factory: Arc::new(rep_fac),
-        })
+        Ok(Self { client, config })
     }
 
     /// Create a reusable [Downloader] which is just an alternate form to use the API.
-    pub fn downloader(&self) -> Downloader<C, RF> {
+    pub fn downloader(&self) -> Downloader<C, NoReporting> {
         Downloader::new(self.clone())
     }
 
+    /// Create a reusable [Downloader] which is just an alternate form to use the API.
+    pub fn downloader_with_reporting<RF: ReporterFactory>(&self, rep_fac: RF) -> Downloader<C, RF> {
+        self.downloader_with_reporting_arc(Arc::new(rep_fac))
+    }
+
+    /// Create a reusable [Downloader] which is just an alternate form to use the API.
+    pub fn downloader_with_reporting_arc<RF: ReporterFactory>(
+        &self,
+        rep_fac: Arc<RF>,
+    ) -> Downloader<C, RF> {
+        Downloader::new_with_reporting_arc(self.clone(), rep_fac)
+    }
+
     /// Download a BLOB range (potentially) concurrently
     ///
     /// Returns a stream of [Chunk](streams::Chunk)s.
@@ -107,7 +126,7 @@ impl<C: CondowClient, RF: ReporterFactory> Condow<C, RF> {
         location: C::Location,
         range: R,
     ) -> Result<ChunkStream, CondowError> {
-        self.download_chunks_internal(location, range, GetSizeMode::Default, NoReporter)
+        self.download_chunks_internal(location, range, GetSizeMode::Default, NoReporting)
             .await
             .map(|o| o.into_stream())
     }
@@ -121,7 +140,7 @@ impl<C: CondowClient, RF: ReporterFactory> Condow<C, RF> {
         range: R,
     ) -> Result<PartStream<ChunkStream>, CondowError> {
         let chunk_stream = self
-            .download_chunks_internal(location, range, GetSizeMode::Default, NoReporter)
+            .download_chunks_internal(location, range, GetSizeMode::Default, NoReporting)
             .await
             .map(|o| o.into_stream())?;
         PartStream::from_chunk_stream(chunk_stream)
@@ -138,13 +157,13 @@ impl<C: CondowClient, RF: ReporterFactory> Condow<C, RF> {
         range: R,
         get_size_mode: GetSizeMode,
         reporter: RP,
-    ) -> Result<Outcome<ChunkStream, RP>, CondowError> {
+    ) -> Result<StreamWithReport<ChunkStream, RP>, CondowError> {
         let range: DownloadRange = range.into();
         range.validate()?;
         let range = if let Some(range) = range.sanitized() {
             range
         } else {
-            return Ok(Outcome::new(ChunkStream::empty(), reporter));
+            return Ok(StreamWithReport::new(ChunkStream::empty(), reporter));
         };
 
         let (inclusive_range, bytes_hint) = match range {
@@ -153,7 +172,7 @@ impl<C: CondowClient, RF: ReporterFactory> Condow<C, RF> {
                 if let Some(range) = or.incl_range_from_size(size) {
                     (range, BytesHint::new_exact(range.len()))
                 } else {
-                    return Ok(Outcome::new(ChunkStream::empty(), reporter));
+                    return Ok(StreamWithReport::new(ChunkStream::empty(), reporter));
                 }
             }
             DownloadRange::Closed(cl) => {
@@ -162,12 +181,12 @@ impl<C: CondowClient, RF: ReporterFactory> Condow<C, RF> {
                     if let Some(range) = cl.incl_range_from_size(size) {
                         (range, BytesHint::new_exact(range.len()))
                     } else {
-                        return Ok(Outcome::new(ChunkStream::empty(), reporter));
+                        return Ok(StreamWithReport::new(ChunkStream::empty(), reporter));
                     }
                 } else if let Some(range) = cl.incl_range() {
                     (range, BytesHint::new_at_max(range.len()))
                 } else {
-                    return Ok(Outcome::new(ChunkStream::empty(), reporter));
+                    return Ok(StreamWithReport::new(ChunkStream::empty(), reporter));
                 }
             }
         };
@@ -182,16 +201,37 @@ impl<C: CondowClient, RF: ReporterFactory> Condow<C, RF> {
         )
         .await?;
 
-        Ok(Outcome { reporter, stream })
+        Ok(StreamWithReport { reporter, stream })
     }
 }
 
-pub struct Outcome<St: Stream, R: Reporter> {
+impl<C> Downloads<C::Location> for Condow<C>
+where
+    C: CondowClient,
+{
+    fn download<'a, R: Into<DownloadRange> + Send + Sync + 'static>(
+        &'a self,
+        location: C::Location,
+        range: R,
+    ) -> BoxFuture<'a, Result<PartStream<ChunkStream>, CondowError>> {
+        Box::pin(self.download(location, range))
+    }
+
+    fn download_chunks<'a, R: Into<DownloadRange> + Send + Sync + 'static>(
+        &'a self,
+        location: C::Location,
+        range: R,
+    ) -> BoxFuture<'a, Result<ChunkStream, CondowError>> {
+        Box::pin(self.download_chunks(location, range))
+    }
+}
+
+pub struct StreamWithReport<St: Stream, R: Reporter> {
     pub stream: St,
     pub reporter: R,
 }
 
-impl<St, R> Outcome<St, R>
+impl<St, R> StreamWithReport<St, R>
 where
     St: Stream,
     R: Reporter,
@@ -209,14 +249,14 @@ where
     }
 }
 
-impl<R> Outcome<ChunkStream, R>
+impl<R> StreamWithReport<ChunkStream, R>
 where
     R: Reporter,
 {
-    pub fn part_stream(self) -> Result<Outcome<PartStream<ChunkStream>, R>, CondowError> {
-        let Outcome { stream, reporter } = self;
+    pub fn part_stream(self) -> Result<StreamWithReport<PartStream<ChunkStream>, R>, CondowError> {
+        let StreamWithReport { stream, reporter } = self;
         let part_stream = PartStream::from_chunk_stream(stream)?;
-        Ok(Outcome::new(part_stream, reporter))
+        Ok(StreamWithReport::new(part_stream, reporter))
     }
 
     pub async fn write_buffer(self, buffer: &mut [u8]) -> Result<usize, CondowError> {
@@ -228,7 +268,7 @@ where
     }
 }
 
-impl<S, R> Outcome<PartStream<S>, R>
+impl<S, R> StreamWithReport<PartStream<S>, R>
 where
     S: Stream<Item = ChunkStreamItem> + Unpin,
     R: Reporter,
